import json
import socket
import urllib.request
import urllib.error
import logging

from defence360agent.contracts.config import Core

logger = logging.getLogger(__name__)


class APIError(Exception):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        if len(args) >= 2:
            _, status_code, *args = args
            self.status_code = status_code
        else:
            self.status_code = None


class API:
    _BASE_URL = Core.API_BASE_URL
    _SOCKET_TIMEOUT = 300

    @staticmethod
    def request(request: urllib.request.Request, json_loads=True):
        try:
            with urllib.request.urlopen(
                request,
                # agent should be able to wait for a while
                # in lb queue before being connected
                timeout=API._SOCKET_TIMEOUT,
            ) as response:
                logger.info('Performed request for url=%s method=%s body=%s '
                            'status=%s',
                            request.full_url, getattr(request, 'method', None),
                            request.data, response.status)
                if response.status != 200:
                    raise APIError(
                        'status code is {}'.format(response.status),
                        response.status)
                plain_response = response.read()
                logger.info('Response=%s ...', plain_response[:50])
                if json_loads:
                    result = json.loads(plain_response.decode())
                else:
                    result = plain_response
                return result
        except (UnicodeDecodeError, json.JSONDecodeError,
                urllib.error.URLError, socket.timeout) as e:
            status_code = getattr(e, "code", None)
            raise APIError(
                'request failed, reason: %s' % (e,),
                status_code
            ) from e
