from logging import getLogger

from defence360agent.rpc_tools.lookup import RootEndpoints, bind
from defence360agent.subsys.panels.hosting_panel import HostingPanel

logger = getLogger(__name__)

__all__ = []


class FeatureManagementNativeEndpoints(RootEndpoints):
    def __init__(self, sink):
        super().__init__(sink)
        self.hosting_panel = HostingPanel()

    @bind("feature-management", "native", "status")
    async def feature_management_native_status(self):
        return {
            "items": {
                "supported": await self.hosting_panel.is_native_feature_management_supported(),  # noqa: E501
                "enabled": await self.hosting_panel.is_native_feature_management_enabled(),  # noqa: E501
            }
        }

    @bind("feature-management", "native", "enable")
    async def feature_management_native_enable(self):
        await self.hosting_panel.enable_native_feature_management()

    @bind("feature-management", "native", "disable")
    async def feature_management_native_disable(self):
        disabled = (
            await self.hosting_panel.disable_native_feature_management()
        )
        if disabled:
            return {
                "items": "Imunify360 package extensions have been removed from all packages"  # noqa: E501
            }
