#!/opt/alt/python38/bin/python3
"""This module import peewee_migrate and apply migrations, for Imunify-AV
it's entrypoint for service"""

import os
import sys
from logging import getLogger

import defence360agent.internals.logger
from defence360agent.application.settings import configure
from defence360agent.contracts.config import Core
from defence360agent.contracts.config import Model as Config
from defence360agent.contracts.config import conf_rel2abs
from defence360agent.subsys import systemd_notifier
from peewee import SqliteDatabase
from peewee_migrate import Router

logger = getLogger(__name__)


def apply_migrations():
    """Apply migrations: restructure db, config files, etc."""

    logger.info("Applying database migrations...")
    systemd_notifier.notify(systemd_notifier.AgentState.MIGRATING)

    router = Router(SqliteDatabase(Config.PATH),
                    migrate_dir=conf_rel2abs("migrations"))
    router.run()


def run(*, start_pkg='defence360agent', configure=configure):
    """Entry point for Imunify-AV service. Apply migrations,
    and then replace process with {start_pkg}.run module."""
    os.umask(Core.FILE_UMASK)
    configure()
    defence360agent.internals.logger.reconfigure()
    systemd_notifier.notify(systemd_notifier.AgentState.READY)
    apply_migrations()
    logger.info("Starting main process...")
    systemd_notifier.notify(systemd_notifier.AgentState.STARTING)
    os.execv(sys.executable,
             [sys.executable, '-m', '{}.run'.format(start_pkg)] + sys.argv[1:])


if __name__ == "__main__":
    run()
