import logging
from subprocess import check_call

from defence360agent.contracts.config import IConfig, LocalConfig
from defence360agent.utils import antivirus_mode

logger = logging.getLogger(__name__)


def migrate(
    migrator,
    database,
    fake=False,
    config_file: IConfig = LocalConfig(),
    **kwargs
):
    """Write your migrations here."""

    if fake:
        return

    config = config_file.config_to_dict()
    if not config:
        return
    DisabledRule = migrator.orm["disabled_rules"]
    # deleting "OSSEC" section
    ossec_ignored_rules = config.pop("OSSEC", {}).get("ignore", [])
    for rule in set(ossec_ignored_rules):
        DisabledRule.create_or_get(
            plugin="ossec", rule_id=str(rule), name="Imported from config"
        )

    # deleting "MOD_SEC_BLOCK_BY_SEVERITY:ignore" field
    modsec_ignored_rules = config.get("MOD_SEC_BLOCK_BY_SEVERITY", {}).pop(
        "ignore", []
    )

    for rule in set(modsec_ignored_rules):
        DisabledRule.create_or_get(
            plugin="modsec", rule_id=str(rule), name="Imported from config"
        )

    DisabledRule.create_or_get(
        plugin="modsec",
        rule_id="214920",
        name="COMODO WAF: Inbound Points|Total Incoming Points; "
        "Known for creating lot of False Positives",
    )

    if antivirus_mode.enabled:
        return

    from im360.subsys.panels.hosting_panel import HostingPanel

    try:
        hp = HostingPanel()
        hp.write_global_disabled_rules(modsec_ignored_rules)
        if hp.REBUILD_HTTPDCONF_CMD:
            check_call(hp.REBUILD_HTTPDCONF_CMD)

        from im360.subsys import web_server

        web_server.graceful_restart_sync()
    except Exception as e:
        logger.warning("Error during syncing disabled rules: %s", str(e))

    config_file.dict_to_config(config, overwrite=True, validate=False)


def rollback(migrator, database, fake=False, **kwargs):
    """Write your rollback migrations here."""
    pass
