import re
import logging
import yaml
from pathlib import Path
from defence360agent.utils import atomic_rewrite, log_error_and_ignore

logger = logging.getLogger(__name__)

_WEBSHIELD_CONFIG_DIR = Path("/etc/imunify360-webshield")
_INVISIBLE_CAPTCHA_CONFIGS = [
    "invisible-captcha.conf.rpmsave",  # centos-like
    "invisible-captcha.conf.backup",  # debian-like
]
OPTION_REGEXP = re.compile(
    r"^\s*set\s+\$invisible_captcha\s+(on|off);", flags=re.MULTILINE
)

_LOCAL_CONFIG_PATH = Path("/etc/sysconfig/imunify360/imunify360.config")


@log_error_and_ignore()
def migrate(
    migrator,
    database,
    fake=False,
    local_config=_LOCAL_CONFIG_PATH,
    nginx_config_dir=_WEBSHIELD_CONFIG_DIR,
    **kwargs
):
    if fake:
        return
    invisible_captcha = False
    for c in _INVISIBLE_CAPTCHA_CONFIGS:
        ic_config_path = nginx_config_dir / c
        if ic_config_path.exists():
            ic_config_content = ic_config_path.read_text()
            m = OPTION_REGEXP.search(ic_config_content)
            if m is None:
                logger.warning(
                    "invisible_captcha setting not found in %s", ic_config_path
                )
            else:
                invisible_captcha = m.group(1) == "on"
            break

    # only set in config if true,
    # because otherwise it is already false (default)
    if invisible_captcha:
        if local_config.exists():
            config = yaml.safe_load(local_config.read_text()) or {}
        else:
            config = {}
        webshield_config = config.setdefault("WEBSHIELD", {})
        webshield_config["invisible_captcha"] = invisible_captcha
        config_text = yaml.dump(config, default_flow_style=False)
        atomic_rewrite(
            str(local_config), config_text, permissions=0o600, backup=False
        )


def rollback(migrator, database, fake=False, **kwargs):
    pass
