from logging import getLogger

from defence360agent.contracts.plugins import MessageSource
from defence360agent.malwarelib.scan.cleaners.detached_dir import (
    MalwareDetachedDirCleaner,
)
from defence360agent.malwarelib.scan.cleaners.outdated_scans import (
    OutdatedScansCleaner,
)
from defence360agent.malwarelib.scan.cleaners.uncompleted_scans import (
    UncompletedScansCleaner,
)
from defence360agent.utils import recurring_check

logger = getLogger(__name__)


class MrProper(MessageSource):
    CLEANERS = [
        MalwareDetachedDirCleaner,
        OutdatedScansCleaner,
        UncompletedScansCleaner,
    ]

    def __init__(self):
        self._loop = None
        self._sink = None
        self._tasks = []

    async def create_source(self, loop, sink):
        self._loop = loop
        self._sink = sink
        for cleaner in self.CLEANERS:
            self._tasks.append(
                self._loop.create_task(
                    recurring_check(cleaner.PERIOD)(cleaner.cleanup)()
                )
            )

    async def shutdown(self):
        for task in self._tasks:
            task.cancel()
        for task in self._tasks:
            # no need to suppress CancelledError because
            # recurring_check() catches it already
            await task
