#!/opt/alt/python38/bin/python3
from defence360agent import rpc_handlers
from defence360agent.application.settings import configure
from defence360agent.feature_management.plugins import native
from defence360agent.feature_management.plugins import (
    watcher as feature_management_watcher,
)
from defence360agent.internals.cln import subscribe_to_license_changes
from defence360agent.internals.iaid import IndependentAgentIDAPI
from defence360agent.malwarelib.plugins import (
    cleanup,
    detached_scan,
    malware_response,
    mrs_suspicious_uploader,
    scan_queue,
    scanner,
    schedule_watcher,
    store,
)
from defence360agent.malwarelib.utils import reset_malware_schedule
from defence360agent.plugins import (
    # use lexicographical order (but don't rely on it in code)
    accumulate,
    check_license,
    client,
    config_merger,
    config_watcher,
    event_hook_executor,
    event_hooks,
    files_recurring_update,
    generic_sensor,
    icontact_sender,
    idle_time_out,
    lve_utils_install,
    malware_ignore_paths,
    mr_proper,
    ping,
    send_server_config,
)
from defence360agent.server import start
from defence360agent.utils import make_coro


def get_plugins():
    """Return plugins in unspecified order."""
    return [
        # use lexicographical order (but don't rely on it in code)
        accumulate.Accumulate,
        check_license.CheckLicense,
        cleanup.Cleanup,
        cleanup.StorageController,
        config_merger.ConfigMerger,
        config_watcher.ConfigWatcher,
        event_hook_executor.EventHookExecutor,
        event_hooks.EventHooks,
        feature_management_watcher.PasswdWatcher,
        files_recurring_update.FilesRecurringUpdateTask,
        idle_time_out.IdleTimeOutCheck,
        lve_utils_install.LveUtilsAutoInstaller,
        malware_response.MalwareResponsePlugin,
        mr_proper.MrProper,
        mrs_suspicious_uploader.SuspiciousToMRSUploader,
        native.NativeEventMonitor,
        ping.SendPing,
        scan_queue.QueueSupervisor,
        scanner.Scanner,
        schedule_watcher.ScheduleWatcher,
    ]


def get_av_only_plugins():
    return [
        # use lexicographical order (but don't rely on it in code)
        cleanup.ResultProcessor,
        client.SendToServer,
        detached_scan.DetachedScanPlugin,
        generic_sensor.GenericSensor,
        icontact_sender.IContactSender,
        send_server_config.SendServerConfig,
        store.StoreMalwareHits,
        malware_ignore_paths.BaseMalwareIgnorePath,
    ]


async def init_actions():
    subscribe_to_license_changes(make_coro(reset_malware_schedule))
    subscribe_to_license_changes(IndependentAgentIDAPI.reactivate)
    await IndependentAgentIDAPI.activate()
    rpc_handlers.init()


def run():
    configure()
    plugins = get_plugins() + get_av_only_plugins()
    start(plugins, init_actions)


if __name__ == '__main__':
    run()
