from defence360agent.rpc_tools.lookup import RootEndpoints, bind
from defence360agent.utils import is_centos6_or_cloudlinux6
from defence360agent.utils.hyperscan import is_ssse3_supported
from defence360agent.utils.resource_limits import (
    LimitsMethod,
    get_current_method,
)


class ConflictsEndpoints(RootEndpoints):
    @bind("3rdparty", "conflicts")
    async def thirdparty_conflicts(self):
        results = {}
        for func in self._funcs:
            await func(results)
        return {"items": results}

    @property
    def _funcs(self):
        """ List of functions for "3rdparty conflicts" endpoint """

        return [self._intensity_conflicts, self._hyperscan_conflicts]

    @staticmethod
    async def _intensity_conflicts(results_dict):
        if await get_current_method() == LimitsMethod.NICE:
            results_dict["scan_intensity_io_cfq"] = True
        return results_dict

    @classmethod
    async def _hyperscan_conflicts(cls, results_dict):
        if not is_ssse3_supported():
            # SSSE3 instruction set is required to run HyperScan
            # We may change it to SSE2 in the future
            results_dict["scan_hyperscan_no_ssse3"] = True

        if is_centos6_or_cloudlinux6():
            # alt-php-hyperscan is not available on CentOS6/CloudLinux6
            results_dict["scan_hyperscan_el6"] = True

        return results_dict
