"""Additional changes, which need to use RPC/CLI schemas. """
from pathlib import Path

import im360.malwarelib.subsys.aibolit
import im360.model.update_hooks
import im360.subsys.ossec
import im360.subsys.panels.update_hooks
import im360.subsys.proactive
import im360.subsys.webshield
from defence360agent.application.settings import configure as base_configure
from defence360agent.model import infected_domain
from im360 import files
from im360.application import tags
from im360.model import (
    country,
    firewall,
    incident,
    messages_to_send,
    proactive,
)
from im360.simple_rpc.schema import init_validator
from im360.simple_rpc.validate import SchemaValidator, validate_middleware


def get_schema_paths():
    path = Path(__file__).parent.parent.absolute()
    schema_paths = (
        path / "simple_rpc",
        path / "malwarelib" / "rpc",
    )
    return schema_paths


def setup_file_hooks() -> None:
    files.Index.add_hook(files.GEO, im360.model.update_hooks.update_geodb)
    files.Index.add_hook(
        files.MODSEC, im360.subsys.panels.update_hooks.update_vendors
    )
    files.Index.add_hook(files.OSSEC, im360.subsys.ossec.on_files_update)
    files.Index.add_hook(
        files.IP_RECORD, im360.subsys.panels.update_hooks.update_iprecord
    )
    files.Index.add_hook(
        files.PHP_IMMUNITY, im360.subsys.proactive.update_hook
    )
    files.Index.add_hook(
        files.PROACTIVE, im360.subsys.proactive.recreate_signatures_on_update
    )
    files.Index.add_hook(
        files.SIGS,
        im360.malwarelib.subsys.aibolit.restart_on_sigs_or_config_update,
    )
    files.Index.add_hook(
        files.REALTIME_AV_CONF,
        im360.malwarelib.subsys.aibolit.restart_on_sigs_or_config_update,
    )


def configure():
    base_configure(
        init_validator=init_validator,
        validator_cls=SchemaValidator,
        validate_middleware_wrap=validate_middleware,
        schema_paths=get_schema_paths(),
        models_modules=[
            country,
            firewall,
            incident,
            infected_domain,
            messages_to_send,
            proactive,
        ],
        set_sentry_tags=tags.fill,
    )
    files.configure()
    setup_file_hooks()
