from typing import Dict, Set

from im360.utils.net import is_ipv6_enabled
from im360.utils.validate import IP

_state = {}  # type: Dict[str, bool]


def init() -> None:
    global _state
    _state = {IP.V4: True, IP.V6: is_ipv6_enabled()}


def enable(ip_version) -> None:
    _state[ip_version] = True


def disable(ip_version) -> bool:
    if ip_version == IP.V6:
        _state[ip_version] = False
        return True
    return False


def is_enabled(ip_version) -> bool:
    return _state[ip_version]


def enabled() -> Set[str]:
    return set(k for k, v in sorted(_state.items()) if v)


def all() -> Set[str]:
    return set(_state.keys())
