import asyncio

from defence360agent.model import instance
from defence360agent.model.simplification import run_in_executor
from im360.model.geoip_data import GeoCSVReader

from .country import Country


async def update_geodb(_, is_updated):
    """
    Load countries list (if it is empty) from the csv file on update.
    """
    def update_country_db_from_csv():
        csv = GeoCSVReader()
        with instance.db.atomic():
            Country.update_from(csv.countries())

    loop = asyncio.get_event_loop()

    # Check that table country is not empty
    countries_count = await run_in_executor(
        loop,
        lambda: (Country.select().count())
    )

    if (countries_count == 0) or is_updated:
        await run_in_executor(loop, update_country_db_from_csv)
