from logging import getLogger

from defence360agent.contracts.config import Malware as Config
from defence360agent.utils import recurring_check
from defence360agent.contracts.plugins import (
    MessageSink,
    MessageSource,
)
from im360.malwarelib.subsys import pure_ftpd
from im360.subsys.panels import hosting_panel

logger = getLogger(__name__)


class Conflicts(MessageSource, MessageSink):
    def __init__(self):
        self._loop = None
        self._sink = None
        self._pure_check_task = None

    async def create_sink(self, loop):
        pass

    async def create_source(self, loop, sink):
        self._loop = loop
        self._sink = sink

        self._pure_check_task = self._loop.create_task(
            recurring_check(Config.CONFLICTS_CHECK_PERIOD)(self._check_pure)()
            )

    async def shutdown(self):
        if await pure_ftpd.uploadscript_status():
            panel = hosting_panel.HostingPanel()
            await pure_ftpd.disable_purescan(panel)

        for t in [self._pure_check_task]:
            t.cancel()
            await t

    @staticmethod
    async def _check_pure():
        if not pure_ftpd.detect():
            return  # pure-ftpd is not installed

        # ftp scan integration is enabled in config
        active_in_config = Config.PURE_SCAN
        # should ftp scan integration be enabled
        enabled = active_in_config and not Config.INOTIFY_ENABLED
        running = await pure_ftpd.uploadscript_status()
        thirdparty_uploadscript = pure_ftpd.thirdparty_uploadscript()

        if thirdparty_uploadscript is not None:
            if running:
                logger.info("Third-party pure-uploadscript detected: %s,"
                            " disabling pure-ftpd scans",
                            thirdparty_uploadscript)
                await pure_ftpd.uploadscript_disable()
            return

        panel = hosting_panel.HostingPanel()
        if enabled:
            if not running:
                logger.info("Enabling pure-ftpd service")
                await pure_ftpd.uploadscript_enable()
            if not pure_ftpd.scan_in_config_enabled(panel):
                logger.info("Enable upload script in pure-ftpd configs")
                await pure_ftpd.enable_scan_in_config(panel)
            # active & enabled & configured & running
            logger.info("FTP scan is enabled")
        elif running and not enabled:
            logger.info("Disabling pure-ftpd scans")
            await pure_ftpd.disable_purescan(panel)
