from logging import getLogger
from typing import Dict

from defence360agent.contracts.messages import MalwareCleanup
from defence360agent.contracts.plugins import MessageSink, MessageSource, \
    expect
from defence360agent.malwarelib.config import MalwareHitStatus
from defence360agent.malwarelib.model import MalwareHit

logger = getLogger(__name__)


class PostAction(MessageSink, MessageSource):
    PROCESSING_ORDER = MessageSink.ProcessingOrder.POST_ACTION

    async def create_sink(self, loop):
        pass

    async def create_source(self, loop, sink):
        self._sink = sink

    @expect(MalwareCleanup)
    async def process_post_action(self, message: Dict):
        post_action = message.pop('post_action', None)
        if not post_action:
            return

        scan_id = message['scan_id']
        cause = message['cause']
        initiator = message['initiator']

        found_hits = [hit for hit in message['hits']
                      if hit.status == MalwareHitStatus.FOUND]
        other_hits = [hit for hit in message['hits']
                      if hit.status != MalwareHitStatus.FOUND]

        if await post_action(hits=found_hits, scan_id=scan_id, cause=cause,
                             initiator=initiator, sink=self._sink):
            message['hits'] = MalwareHit.refresh_hits(found_hits) + other_hits

        return message
