import logging

from defence360agent.contracts.messages import Reject
from defence360agent.contracts.plugins import MessageSink, expect
from im360.contracts.config import Webshield
from im360.contracts.messages import (
    CaptchaDosAlert, SensorAlert, SynclistResponse)
from im360.model.firewall import IPList

logger = logging.getLogger(__name__)


class SuppressGraylistedProcessing(MessageSink):
    PROCESSING_ORDER = MessageSink.ProcessingOrder.IGNORE_MESSAGE

    async def create_sink(self, loop):
        "MessageSink method"

    @property
    def is_graylist_enabled(self):
        return Webshield.ENABLE

    @expect(SensorAlert, CaptchaDosAlert)
    async def suppress_message(self, message):
        if not self.is_graylist_enabled:
            raise Reject("Webshield is disabled.")

    @expect(SynclistResponse)
    async def remove_graylisted_from_blocklist(self, message):
        """
        Keep blacklisted ips in the blocklist if webshield is disabled.
        Save gray splash_screen ips to DB regardless of whether
        webshield/splash_screen is enabled or not.
        """
        if (not self.is_graylist_enabled and
                isinstance(message["blocklist"], dict)):
            blocklist = {
                ip: value
                for ip, value in message["blocklist"].items()
                if IPList.action_type2listname(value.get(IPList.ACTION_TYPE))
                in [IPList.BLACK, IPList.GRAY_SPLASHSCREEN]
            }

            message['blocklist'] = blocklist
