from defence360agent.contracts.messages import Reject
from defence360agent.contracts.plugins import (
    expect,
    MessageSink,
    MessageSource,
)
from im360.contracts.messages import (
    ServerPush,
    SynclistResponse,
)


class ServerPushToSynclist(MessageSink, MessageSource):
    """
    Is used to convert ServerPush message to SyncList
    to avoid the need to maintain handlers for both type messages.
    """
    PROCESSING_ORDER = MessageSink.ProcessingOrder.PRE_PROCESS_MESSAGE

    def __init__(self):
        self._sink = None

    async def create_sink(self, loop):
        "MessageSink method"

    async def create_source(self, loop, sink):
        self._sink = sink

    @expect(ServerPush)
    async def handle_server_push(self, message):
        """Convert ServerPush into Synclist"""
        lists = {}
        for name, ignored_method in [
            ("blocklist", "UNBLOCK"),
            ("unblocklist", "BLOCK"),
        ]:
            d = lists[name] = {}
            if message["method"] != ignored_method:
                d[message["attackers_ip"]] = message.get("properties")
        msg = SynclistResponse(method="SYNCLIST", **lists)
        await self._sink.process_message(msg)
        raise Reject("To handle ServerPush SyncList will be used")
