import logging

from defence360agent.contracts.messages import UpdateFiles
from defence360agent.contracts.plugins import MessageSink, expect
from im360 import files

logger = logging.getLogger(__name__)


PHP_IMMUNITY_FILENAME = "/usr/share/i360-php-opts/autorules.yaml"


class UpdateFilesOnServerRequest(MessageSink):
    async def create_sink(self, loop):
        self._loop = loop

    @expect(UpdateFiles, list_type=files.MODSEC)
    @expect(UpdateFiles, list_type=files.IP_RECORD)
    async def process_update_request(self, message):
        logger.info(
            "%s files will be updated on server request", message.list_type
        )
        await files.update_and_log_error(message.list_type)

    @expect(UpdateFiles, list_type=files.PHP_IMMUNITY)
    @expect(UpdateFiles, list_type=files.PROACTIVE)
    async def process_update_request_force(self, message):
        logger.info(
            "%s files will be force updated on server request",
            message.list_type,
        )
        await files.update_and_log_error(message.list_type, force=True)
