import os


class ScanFailedError(Exception):
    def __init__(
        self,
        *,
        message,
        command,
        return_code,
        out,
        err,
        scan_id,
        path,
    ):
        """
        In some scanners out and errors can be lists of bytes or bytes, we will
        convert all to string
        :param message:
        :param command:
        :param return_code:
        :param out:
        :param err:
        """
        self.message = message
        self.command = command
        self.return_code = return_code
        self.out = self._convert_to_str(out)
        self.err = self._convert_to_str(err)
        self.path = path
        self.scan_id = scan_id

    def __str__(self):
        return (
            "Scan failed with {}: command: {} return code: {} out: {} "
            "err: {}".format(
                self.message,
                self.command,
                self.return_code,
                self.out,
                self.err,
            )
        )

    @staticmethod
    def _convert_to_str(obj):
        if type(obj) == str:
            return obj
        elif type(obj) == bytes:
            return os.fsdecode(obj)
        elif type(obj) == list:
            return "".join(
                [
                    elem.decode() if type(elem) == bytes else elem
                    for elem in obj
                ]
            )
        elif obj is None:
            return ""
        else:
            raise TypeError("Unsupported type to convert to string")


class ScanAlreadyCompleteError(Exception):
    pass


class ScanInfoError(Exception):
    pass
