import os
import shutil
from datetime import datetime, timedelta
from logging import getLogger

from defence360agent.malwarelib.scan.ai_bolit.detached import (
    AiBolitDetachedDir,
)
from defence360agent.mr_proper import BaseCleaner
from defence360agent.utils import antivirus_mode

logger = getLogger(__name__)


class MalwareDetachedDirCleaner(BaseCleaner):
    PERIOD = timedelta(days=1).total_seconds()
    CLEANUP_LIMIT_DELTA = timedelta(days=30)

    if antivirus_mode.enabled:
        _DETACHED_DIRS = [
            AiBolitDetachedDir,
        ]
    else:
        from im360.malwarelib.scan.mds.detached import (
            MDSDetachedCleanupDir,
            MDSDetachedRestoreDir,
            MDSDetachedScanDir,
        )

        _DETACHED_DIRS = [
            AiBolitDetachedDir,
            MDSDetachedCleanupDir,
            MDSDetachedRestoreDir,
            MDSDetachedScanDir,
        ]

    @classmethod
    async def cleanup(cls):
        now = datetime.now()
        time_limit = (now - cls.CLEANUP_LIMIT_DELTA).timestamp()
        for dir_cls in cls._DETACHED_DIRS:
            detached_directory = dir_cls.DETACHED_DIR
            if os.path.exists(detached_directory):
                entry: os.DirEntry
                for entry in os.scandir(detached_directory):
                    if entry.is_dir() and entry.stat().st_mtime < time_limit:
                        shutil.rmtree(entry.path)
                        logger.info(
                            "Cleaned %s outdated operation dir", entry.name
                        )
