from pathlib import Path


def trim(message: str, *, max_length: int) -> str:
    """
    Remove the middle part of the string,
    so that the overall length is no more than specified.

    Returned string's length should not exceed max_length.
    """
    # We don't want to handle corner cases.
    # If you need to trim strings that much use another function.
    assert (
        max_length > 50
    ), "Specify a bigger max_length or don't use this function"

    if len(message) <= max_length:
        return message

    filler = "\n...\n"
    left_cutoff = max_length // 2
    right_cutoff = max(1, max_length - left_cutoff - len(filler))
    return message[:left_cutoff] + filler + message[-right_cutoff:]


def trim_file_content(log_path: Path) -> str:
    # 4000 is 50 (~ screen height worth of text) lines of 80 characters
    try:
        log = trim(
            log_path.read_text(encoding="utf-8", errors="backslashreplace"),
            max_length=4000,
        )
    except FileNotFoundError:
        log = ""
    return log
