import hashlib
import os
from datetime import datetime, timedelta

from defence360agent.contracts.config import (
    ConfigFile,
    MalwareScanSchedule,
    MalwareScanScheduleInterval,
)
from defence360agent.contracts.license import LicenseCLN


def get_memory(intensity_ram):
    return str(intensity_ram) + 'M'


def hash_path(full_path: str, hash_provider=hashlib.sha256) -> str:
    """Calculate hash from full path"""
    hash_object = hash_provider()
    hash_object.update(os.fsencode(full_path))
    return hash_object.hexdigest()


def reset_malware_schedule(config_file_path=None):
    """
    When ImunifyAV+ falls to free license, we should
    reset scheduled scanner settings to monthly mode
    (daily and weekly modes are not supported in free version)
    """
    if LicenseCLN.is_valid_av_plus():
        return
    if MalwareScanSchedule.INTERVAL not in (
            MalwareScanScheduleInterval.WEEK, MalwareScanScheduleInterval.DAY
    ):
        return
    tomorrow = (datetime.today() + timedelta(days=1))
    next_day = tomorrow.day
    new_data = {
        'MALWARE_SCAN_SCHEDULE': {
            'interval': 'month',
            'day_of_month': next_day,
            'hour': 3,
        }
    }

    ConfigFile(path=config_file_path).dict_to_config(new_data)
