import asyncio
import logging

from defence360agent.contracts.license import LicenseError
from defence360agent.malwarelib.utils import malware_response

logger = logging.getLogger(__name__)

FALSE_POSITIVE = 'fp'
FALSE_NEGATIVE = 'fn'


async def submit_malware(filename, type, reason=None):
    """
    Submit file to MRS for analysis
    :param filename: str -> path to file to submit
    :param type: str -> report type (fp or fn)
    :param scanner: int -> scanner id
    :param reason: str -> malware name provided by scanner
    :raises LicenseError:
    """
    # submit to MRS
    upload_reason = (
        malware_response.FALSE_POSITIVE
        if type == FALSE_POSITIVE
        else malware_response.FALSE_NEGATIVE
    )
    try:
        await malware_response.upload_file(filename, upload_reason)
    except malware_response.ClientError as e:
        logger.error("File %s uploading failed. %s.", filename, e)


def submit_in_background(*args, **kwargs):
    def report_error(future):
        e = future.exception()
        if isinstance(e, LicenseError):
            logger.warning(str(e))
        elif e is not None:
            logger.exception("File submission failed", exc_info=e)

    loop = asyncio.get_event_loop()
    task = loop.create_task(submit_malware(*args, **kwargs))
    task.add_done_callback(report_error)
