"""MalwareIgnorePath plugin will update malware paths filters.

It listens for MalwareIgnorePathUpdated message, and calls
realtime_av.update() on receiving it. This call also is performed on
plugin initialization.
"""
import asyncio
import logging

from defence360agent.contracts.plugins import MessageSink, expect
from defence360agent.contracts.messages import (
    FilesUpdated,
    MalwareIgnorePathUpdated
)
from defence360agent.subsys import realtime_av
from defence360agent.files import REALTIME_AV_CONF

logger = logging.getLogger(__name__)


class BaseMalwareIgnorePath(MessageSink):
    def __init__(self):
        pass

    async def create_sink(self, loop) -> None:
        self._loop = loop
        await self.call_update()

    async def shutdown(self) -> None:
        pass

    @expect(MalwareIgnorePathUpdated)
    @expect(FilesUpdated, files_type=REALTIME_AV_CONF)
    async def on_message(self, message: MalwareIgnorePathUpdated) -> None:
        await self.call_update()

    async def call_update(self) -> None:
        try:
            realtime_av.generate_configs()
            self.reload_services()
        except asyncio.CancelledError:
            raise
        except Exception as e:
            logger.warning("failed to update malware path filters: %s", e)

    def reload_services(self):
        pass
