from typing import Optional, Union

from defence360agent.contracts.plugins import expect
from defence360agent.malwarelib.config import MalwareScanResourceType
from defence360agent.malwarelib.plugins.detached_scan import (
    DetachedScanPlugin as DetachedScanPluginAV,
)
from defence360agent.malwarelib.plugins.scan_queue import QueueSupervisorSync
from defence360agent.malwarelib.scan.ai_bolit.detached import (
    AiBolitDetachedScan,
)
from im360.contracts.messages import MalwareDatabaseScan
from im360.malwarelib.scan.mds.detached import MDSDetachedScan


class DetachedScanPlugin(DetachedScanPluginAV):
    @staticmethod
    def _get_detached_scan(
        resource_type: Optional[Union[str, MalwareScanResourceType]], scan_id
    ):
        if resource_type is not None and (
            MalwareScanResourceType(resource_type)
            is MalwareScanResourceType.DB
        ):
            return MDSDetachedScan(scan_id)
        return AiBolitDetachedScan(scan_id)

    @expect(MalwareDatabaseScan)
    async def complete_scan_db(self, message):
        queued_scan = QueueSupervisorSync.queue.find(scanid=message["scan_id"])
        if queued_scan:
            QueueSupervisorSync.queue.remove(queued_scan)
            await self._recheck_scan_queue()
