import logging

from defence360agent.rpc_tools import ValidationError
from defence360agent.rpc_tools.lookup import RootEndpoints, bind
from im360.malwarelib.subsys import pure_ftpd
from im360.subsys.panels import hosting_panel


logger = logging.getLogger(__name__)


class PureFTPDEndpoints(RootEndpoints):
    @bind("pure-ftpd", "enable")
    async def enable_pure_ftpd(self):
        if not pure_ftpd.detect():
            raise ValidationError("pure-ftpd is not installed")

        running = await pure_ftpd.uploadscript_status()
        thirdparty_uploadscript = pure_ftpd.thirdparty_uploadscript()

        if thirdparty_uploadscript is not None:
            raise ValidationError(
                "Third-party pure-uploadscript detected: {},"
                " disabling pure-ftpd scans".format(thirdparty_uploadscript)
            )

        panel = hosting_panel.HostingPanel()
        if not running:
            logger.info("Enabling pure-ftpd service")
            await pure_ftpd.uploadscript_enable()
        if not pure_ftpd.scan_in_config_enabled(panel):
            logger.info("Enabling upload script in pure-ftpd configs")
            await pure_ftpd.enable_scan_in_config(panel)
