import logging

import psutil

from defence360agent.utils import retry_on

logger = logging.getLogger(__name__)


@retry_on(FileNotFoundError, max_tries=10)
def cxswatch_running() -> bool:
    """Whether cxswatch process is running."""
    it = psutil.process_iter(attrs=["name"])
    while True:
        try:
            proc = next(it)
        except StopIteration:
            break
        except FileNotFoundError as e:
            logger.warning("File not found during process iter: %s", e)
            raise
        else:
            if proc.info["name"] == "cxswatch":
                return True
    return False
