import asyncio
import datetime
from logging import getLogger
from typing import Tuple

from defence360agent.malwarelib.config import MalwareScanType
from defence360agent.malwarelib.scan.scanner import MalwareScanner
from im360.contracts.config import BackupRestore
from restore_infected.restore import async_restore_infected

logger = getLogger(__name__)


def _get_scan_function(sink, tmpdir=None):
    async def _custom_scan_function(files: list) -> list:
        """Used for checking files from backups before their recovering"""
        if not files:
            return []
        still_infected = []
        scanner = MalwareScanner(sink=sink, tmpdir=tmpdir)
        scanner.start(
            files,
            scan_type=MalwareScanType.RESTORE_FROM_BACKUP,
            use_filters=False,
        )
        result = await scanner.async_wait()
        if result is not None:
            still_infected = [r["file_name"] for r in result["results"]]
        return still_infected

    return _custom_scan_function


async def restore_files(
    files: list, until: datetime, sink, tmp_dir=None
) -> Tuple[list, list]:

    backend = BackupRestore.backup_system()
    assert backend, "Backup system is not selected!"
    restored, not_restored = [], files

    try:
        scan_function = _get_scan_function(sink)
        restored, not_restored = await async_restore_infected(
            backend=backend,
            files=files,
            until=until,
            scan_func=scan_function,
            tmp_dir=tmp_dir,
        )
    except asyncio.CancelledError:
        raise
    except Exception as e:
        logger.exception("Error during restore from backup")
    return restored, not_restored
