"""MalwareIgnorePath plugin will update malware paths filters.

It listens for MalwareIgnorePathUpdated message, and calls
realtime_av.update() on receiving it. This call also is performed on
plugin initialization.
"""
import asyncio
from contextlib import suppress

from defence360agent.plugins.malware_ignore_paths import BaseMalwareIgnorePath
from im360.subsys import realtime_av


class MalwareIgnorePath(BaseMalwareIgnorePath):
    def __init__(self):
        self._reload_task = None
        super().__init__()

    def reload_services(self):
        self._reload_task = self._loop.create_task(
            realtime_av.reload_services()
        )

    async def shutdown(self):  # pragma: no cover
        if self._reload_task:
            self._reload_task.cancel()
            with suppress(asyncio.CancelledError):
                await self._reload_task
