import asyncio
from functools import partial
from pathlib import Path

from defence360agent.contracts.config import Malware, PermissionsConfig
from defence360agent.contracts.messages import RestoreFromBackupTask
from defence360agent.feature_management.constants import AV, AV_REPORT, FULL
from defence360agent.feature_management.lookup import feature
from defence360agent.malwarelib.config import (
    MalwareHitStatus,
    MalwareScanResourceType,
)
from defence360agent.malwarelib.model import MalwareHit
from defence360agent.malwarelib.plugins.scan_queue import (
    QueueSupervisorSync as ScanQueue,
)
from defence360agent.malwarelib.utils.endpoints import MaliciousEndpointStatus
from defence360agent.malwarelib.utils.submit import FALSE_POSITIVE
from defence360agent.model.simplification import run_in_executor
from defence360agent.rpc_tools.lookup import CommonEndpoints, bind
from im360.malwarelib.subsys.malware import HackerTrapHitsSaver, MalwareAction


class MalwareEndpoints(CommonEndpoints):
    def __init__(self, sink):
        super().__init__(sink)
        self.queue = ScanQueue(sink=sink)

    @feature(AV, [FULL, AV_REPORT])
    @bind('malware', 'malicious', 'restore-from-backup')
    async def restore_from_backup(self, ids, user=None) -> None:
        """Sends RestoreFromBackupTask message and releases the endpoint."""
        hits = MalwareHit.malicious_select(ids, user=user)
        if hits:
            await self._sink.process_message(
                RestoreFromBackupTask(hits=hits, initiator=user)
            )
